﻿/******************* 
 * Blind-Spot Test *
 *******************/

import { core, data, sound, util, visual, hardware } from './lib/psychojs-2023.1.3.js';
const { PsychoJS } = core;
const { TrialHandler, MultiStairHandler } = data;
const { Scheduler } = util;
//some handy aliases as in the psychopy scripts;
const { abs, sin, cos, PI: pi, sqrt } = Math;
const { round } = util;


// store info about the experiment session:
let expName = 'blind-spot';  // from the Builder filename that created this script
let expInfo = {
    'participant': '',
    'session': '',
};

// Start code blocks for 'Before Experiment'
// init psychoJS:
const psychoJS = new PsychoJS({
  debug: true
});

// open window:
psychoJS.openWindow({
  fullscr: true,
  color: new util.Color([0, 0, 0]),
  units: 'pix',
  waitBlanking: true
});
// schedule the experiment:
psychoJS.schedule(psychoJS.gui.DlgFromDict({
  dictionary: expInfo,
  title: expName
}));

const flowScheduler = new Scheduler(psychoJS);
const dialogCancelScheduler = new Scheduler(psychoJS);
psychoJS.scheduleCondition(function() { return (psychoJS.gui.dialogComponent.button === 'OK'); }, flowScheduler, dialogCancelScheduler);

// flowScheduler gets run if the participants presses OK
flowScheduler.add(updateInfo); // add timeStamp
flowScheduler.add(experimentInit);
flowScheduler.add(screen_scaleRoutineBegin());
flowScheduler.add(screen_scaleRoutineEachFrame());
flowScheduler.add(screen_scaleRoutineEnd());
flowScheduler.add(blind_spot_RRoutineBegin());
flowScheduler.add(blind_spot_RRoutineEachFrame());
flowScheduler.add(blind_spot_RRoutineEnd());
flowScheduler.add(blind_spot_LRoutineBegin());
flowScheduler.add(blind_spot_LRoutineEachFrame());
flowScheduler.add(blind_spot_LRoutineEnd());
flowScheduler.add(rectangleRoutineBegin());
flowScheduler.add(rectangleRoutineEachFrame());
flowScheduler.add(rectangleRoutineEnd());
flowScheduler.add(endRoutineBegin());
flowScheduler.add(endRoutineEachFrame());
flowScheduler.add(endRoutineEnd());
flowScheduler.add(quitPsychoJS, '', true);

// quit if user presses Cancel in dialog box:
dialogCancelScheduler.add(quitPsychoJS, '', false);

psychoJS.start({
  expName: expName,
  expInfo: expInfo,
  resources: [
    // resources:
    {'name': 'creditCard.png', 'path': 'https://pavlovia.org/assets/default/creditCard.png'},
    {'name': 'eyes-5809350_1280.png', 'path': 'eyes-5809350_1280.png'},
  ]
});

psychoJS.experimentLogger.setLevel(core.Logger.ServerLevel.EXP);


var currentLoop;
var frameDur;
async function updateInfo() {
  currentLoop = psychoJS.experiment;  // right now there are no loops
  expInfo['date'] = util.MonotonicClock.getDateStr();  // add a simple timestamp
  expInfo['expName'] = expName;
  expInfo['psychopyVersion'] = '2023.1.3';
  expInfo['OS'] = window.navigator.platform;


  // store frame rate of monitor if we can measure it successfully
  expInfo['frameRate'] = psychoJS.window.getActualFrameRate();
  if (typeof expInfo['frameRate'] !== 'undefined')
    frameDur = 1.0 / Math.round(expInfo['frameRate']);
  else
    frameDur = 1.0 / 60.0; // couldn't get a reliable measure so guess

  // add info from the URL:
  util.addInfoFromUrl(expInfo);
  

  
  psychoJS.experiment.dataFileName = (("." + "/") + `data/${expInfo["participant"]}_${expName}_${expInfo["date"]}`);


  return Scheduler.Event.NEXT;
}


var screen_scaleClock;
var myClock;
var red;
var white;
var blue;
var transparent;
var oldt;
var x_size;
var y_size;
var screen_height;
var x_scale;
var y_scale;
var dbase;
var unittext;
var vsize;
var text_top;
var text_bottom;
var ccimage;
var blind_spot_RClock;
var text_top_R;
var spot;
var head;
var fixation_cross;
var wink;
var blind_spot_LClock;
var text_top_L;
var head_2;
var wink_2;
var rectangleClock;
var text;
var polygon;
var key_resp;
var spot_L;
var spot_R;
var circle_10;
var endClock;
var text_end;
var globalClock;
var routineTimer;
async function experimentInit() {
  // Initialize components for Routine "screen_scale"
  screen_scaleClock = new util.Clock();
  // Run 'Begin Experiment' code from code_both
  myClock = new util.Clock();
  red = new util.Color([1,0,0])
  white = new util.Color([1,1,1])
  blue = new util.Color([0,0,1])
  transparent = null
  // Run 'Begin Experiment' code from code_scale
  oldt = 0;
  x_size = 8.56;
  y_size = 5.398;
  screen_height = 0;
  if ((psychoJS.window.units === "norm")) {
      x_scale = 0.05;
      y_scale = 0.1;
      dbase = 0.0001;
      unittext = " norm units";
      vsize = 2;
  } else {
      if ((psychoJS.window.units === "pix")) {
          x_scale = 60;
          y_scale = 40;
          dbase = 0.1;
          unittext = " pixels";
          vsize = psychoJS.window.size[1];
      } else {
          x_scale = 0.05;
          y_scale = 0.05;
          dbase = 0.0001;
          unittext = " height units";
          vsize = 1;
      }
  }
  
  text_top = new visual.TextStim({
    win: psychoJS.window,
    name: 'text_top',
    text: 'Resize this image to match the size of a credit card\nUp arrow for taller\nDown arrow for shorter\nLeft arrow for narrower\nRight arrow for wider',
    font: 'Arial',
    units: 'norm', 
    pos: [0, 0.7], height: 0.075,  wrapWidth: 1.5, ori: 0,
    languageStyle: 'LTR',
    color: new util.Color('white'),  opacity: 1,
    depth: -2.0 
  });
  
  text_bottom = new visual.TextStim({
    win: psychoJS.window,
    name: 'text_bottom',
    text: 'Press the space bar when done',
    font: 'Arial',
    units: 'norm', 
    pos: [0, (- 0.6)], height: 0.075,  wrapWidth: 1.5, ori: 0,
    languageStyle: 'LTR',
    color: new util.Color('white'),  opacity: 1,
    depth: -3.0 
  });
  
  ccimage = new visual.ImageStim({
    win : psychoJS.window,
    name : 'ccimage', units : undefined, 
    image : 'creditCard.png', mask : undefined,
    anchor : 'center',
    ori : 0, pos : [0, 0], size : [(x_size * x_scale), (y_size * y_scale)],
    color : new util.Color([1, 1, 1]), opacity : 1,
    flipHoriz : false, flipVert : false,
    texRes : 128, interpolate : true, depth : -4.0 
  });
  // Initialize components for Routine "blind_spot_R"
  blind_spot_RClock = new util.Clock();
  text_top_R = new visual.TextStim({
    win: psychoJS.window,
    name: 'text_top_R',
    text: 'Close your LEFT eye\nWatch the cross\nHold down the space bar whenever the dot is invisible',
    font: 'Arial',
    units: 'norm', 
    pos: [0, 0.7], height: 0.075,  wrapWidth: 1.5, ori: 0,
    languageStyle: 'LTR',
    color: new util.Color('white'),  opacity: 1,
    depth: -1.0 
  });
  
  spot = new visual.Polygon({
    win: psychoJS.window, name: 'spot', 
    edges: 100, size:[1.0, 1.0],
    ori: 0.0, pos: [0, 0],
    anchor: 'center',
    lineWidth: 1.0, 
    colorSpace: 'rgb',
    lineColor: new util.Color('white'),
    fillColor: new util.Color('white'),
    opacity: undefined, depth: -2, interpolate: true,
  });
  
  head = new visual.Polygon({
    win: psychoJS.window, name: 'head', 
    edges: 100, size:[1.0, 1.0],
    ori: 0.0, pos: [0, 0],
    anchor: 'center',
    lineWidth: 1.0, 
    colorSpace: 'rgb',
    lineColor: new util.Color('white'),
    fillColor: new util.Color('white'),
    opacity: undefined, depth: -3, interpolate: true,
  });
  
  fixation_cross = new visual.ShapeStim ({
    win: psychoJS.window, name: 'fixation_cross', 
    vertices: [[-[1.0, 1.0][0]/2.0, -[1.0, 1.0][1]/2.0], [+[1.0, 1.0][0]/2.0, -[1.0, 1.0][1]/2.0], [0, [1.0, 1.0][1]/2.0]],
    ori: 0.0, pos: [0, 0],
    anchor: 'center',
    lineWidth: 1.0, 
    colorSpace: 'rgb',
    lineColor: new util.Color('black'),
    fillColor: new util.Color('black'),
    opacity: 1.0, depth: -4, interpolate: true,
  });
  
  wink = new visual.ImageStim({
    win : psychoJS.window,
    name : 'wink', units : undefined, 
    image : 'eyes-5809350_1280.png', mask : undefined,
    anchor : 'center',
    ori : 0.0, pos : [0, 0], size : 1.0,
    color : new util.Color([1,1,1]), opacity : undefined,
    flipHoriz : true, flipVert : false,
    texRes : 128.0, interpolate : true, depth : -5.0 
  });
  // Initialize components for Routine "blind_spot_L"
  blind_spot_LClock = new util.Clock();
  text_top_L = new visual.TextStim({
    win: psychoJS.window,
    name: 'text_top_L',
    text: 'Close your RIGHT eye\nWatch the cross\nHold down the space bar whenever the dot is invisible',
    font: 'Arial',
    units: 'norm', 
    pos: [0, 0.7], height: 0.075,  wrapWidth: 1.5, ori: 0,
    languageStyle: 'LTR',
    color: new util.Color('white'),  opacity: 1,
    depth: -1.0 
  });
  
  head_2 = new visual.Polygon({
    win: psychoJS.window, name: 'head_2', 
    edges: 100, size:[1.0, 1.0],
    ori: 0.0, pos: [0, 0],
    anchor: 'center',
    lineWidth: 1.0, 
    colorSpace: 'rgb',
    lineColor: new util.Color('white'),
    fillColor: new util.Color('white'),
    opacity: undefined, depth: -2, interpolate: true,
  });
  
  wink_2 = new visual.ImageStim({
    win : psychoJS.window,
    name : 'wink_2', units : undefined, 
    image : 'eyes-5809350_1280.png', mask : undefined,
    anchor : 'center',
    ori : 0.0, pos : [0, 0], size : 1.0,
    color : new util.Color([1,1,1]), opacity : undefined,
    flipHoriz : false, flipVert : false,
    texRes : 128.0, interpolate : true, depth : -3.0 
  });
  // Initialize components for Routine "rectangle"
  rectangleClock = new util.Clock();
  text = new visual.TextStim({
    win: psychoJS.window,
    name: 'text',
    text: 'This shape should be a 5 cm square.\nComponent size  (5*x_scale, 5*y_scale) set every repeat.\nThe circle should have a visual angle of 10 degrees.\nPress space to continue',
    font: 'Arial',
    units: 'norm', 
    pos: [0, (- 0.8)], height: 0.075,  wrapWidth: 1.5, ori: 0,
    languageStyle: 'LTR',
    color: new util.Color('white'),  opacity: 1,
    depth: -1.0 
  });
  
  polygon = new visual.Rect ({
    win: psychoJS.window, name: 'polygon', 
    width: [1.0, 1.0][0], height: [1.0, 1.0][1],
    ori: 0, pos: [0, 0],
    anchor: 'center',
    lineWidth: 1, 
    colorSpace: 'rgb',
    lineColor: new util.Color([1, 1, 1]),
    fillColor: new util.Color([1, 1, 1]),
    opacity: 0.5, depth: -2, interpolate: true,
  });
  
  key_resp = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  spot_L = new visual.Polygon({
    win: psychoJS.window, name: 'spot_L', 
    edges: 100, size:[1.0, 1.0],
    ori: 0.0, pos: [0, 0],
    anchor: 'center',
    lineWidth: 1.0, 
    colorSpace: 'rgb',
    lineColor: new util.Color('red'),
    fillColor: new util.Color('red'),
    opacity: undefined, depth: -4, interpolate: true,
  });
  
  spot_R = new visual.Polygon({
    win: psychoJS.window, name: 'spot_R', 
    edges: 100, size:[1.0, 1.0],
    ori: 0.0, pos: [0, 0],
    anchor: 'center',
    lineWidth: 1.0, 
    colorSpace: 'rgb',
    lineColor: new util.Color('red'),
    fillColor: new util.Color('red'),
    opacity: undefined, depth: -5, interpolate: true,
  });
  
  circle_10 = new visual.Polygon({
    win: psychoJS.window, name: 'circle_10', 
    edges: 100, size:[1.0, 1.0],
    ori: 0.0, pos: [0, 0],
    anchor: 'center',
    lineWidth: 5.0, 
    colorSpace: 'rgb',
    lineColor: new util.Color('white'),
    fillColor: new util.Color('white'),
    opacity: 1.0, depth: -6, interpolate: true,
  });
  
  // Initialize components for Routine "end"
  endClock = new util.Clock();
  text_end = new visual.TextStim({
    win: psychoJS.window,
    name: 'text_end',
    text: 'Fin',
    font: 'Arial',
    units: 'norm', 
    pos: [0, 0], height: 0.1,  wrapWidth: 1.8, ori: 0,
    languageStyle: 'LTR',
    color: new util.Color('white'),  opacity: 1,
    depth: 0.0 
  });
  
  // Create some handy timers
  globalClock = new util.Clock();  // to track the time since experiment started
  routineTimer = new util.CountdownTimer();  // to track time remaining of each (non-slip) routine
  
  return Scheduler.Event.NEXT;
}


var t;
var frameN;
var continueRoutine;
var screen_scaleComponents;
function screen_scaleRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //--- Prepare to start Routine 'screen_scale' ---
    t = 0;
    screen_scaleClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // Run 'Begin Routine' code from code_scale
    psychoJS.eventManager.clearEvents();
    
    // keep track of which components have finished
    screen_scaleComponents = [];
    screen_scaleComponents.push(text_top);
    screen_scaleComponents.push(text_bottom);
    screen_scaleComponents.push(ccimage);
    
    for (const thisComponent of screen_scaleComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


var _pj;
var keys;
var dscale;
function screen_scaleRoutineEachFrame() {
  return async function () {
    //--- Loop for each frame of Routine 'screen_scale' ---
    // get current time
    t = screen_scaleClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // Run 'Each Frame' code from code_scale
    var _pj;
    function _pj_snippets(container) {
        function in_es6(left, right) {
            if (((right instanceof Array) || ((typeof right) === "string"))) {
                return (right.indexOf(left) > (- 1));
            } else {
                if (((right instanceof Map) || (right instanceof Set) || (right instanceof WeakMap) || (right instanceof WeakSet))) {
                    return right.has(left);
                } else {
                    return (left in right);
                }
            }
        }
        container["in_es6"] = in_es6;
        return container;
    }
    _pj = {};
    _pj_snippets(_pj);
    keys = psychoJS.eventManager.getKeys();
    if (keys.length) {
        if (((t - oldt) < 0.5)) {
            dscale = (5 * dbase);
            oldt = t;
        } else {
            dscale = dbase;
            oldt = t;
        }
        if ((_pj.in_es6("space", keys) && (t > 1))) {
            continueRoutine = false;
        } else {
            if (_pj.in_es6("up", keys)) {
                y_scale = (util.round(((y_scale + dscale) * 10000)) / 10000);
            } else {
                if (_pj.in_es6("down", keys)) {
                    y_scale = (util.round(((y_scale - dscale) * 10000)) / 10000);
                } else {
                    if (_pj.in_es6("left", keys)) {
                        x_scale = (util.round(((x_scale - dscale) * 10000)) / 10000);
                    } else {
                        if (_pj.in_es6("right", keys)) {
                            x_scale = (util.round(((x_scale + dscale) * 10000)) / 10000);
                        }
                    }
                }
            }
        }
        screen_height = (util.round(((vsize * 10) / y_scale)) / 10);
        text_bottom.text = (((((((("X Scale = " + x_scale.toString()) + unittext) + " per cm, Y Scale = ") + y_scale.toString()) + unittext) + " per cm\nScreen height = ") + screen_height.toString()) + " cm\n\nPress the space bar when done");
        ccimage.size = [(x_size * x_scale), (y_size * y_scale)];
    }
    
    
    // *text_top* updates
    if (t >= 0.0 && text_top.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text_top.tStart = t;  // (not accounting for frame time here)
      text_top.frameNStart = frameN;  // exact frame index
      
      text_top.setAutoDraw(true);
    }

    
    // *text_bottom* updates
    if (t >= 0.0 && text_bottom.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text_bottom.tStart = t;  // (not accounting for frame time here)
      text_bottom.frameNStart = frameN;  // exact frame index
      
      text_bottom.setAutoDraw(true);
    }

    
    // *ccimage* updates
    if (t >= 0.0 && ccimage.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      ccimage.tStart = t;  // (not accounting for frame time here)
      ccimage.frameNStart = frameN;  // exact frame index
      
      ccimage.setAutoDraw(true);
    }

    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of screen_scaleComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function screen_scaleRoutineEnd(snapshot) {
  return async function () {
    //--- Ending Routine 'screen_scale' ---
    for (const thisComponent of screen_scaleComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // Run 'End Routine' code from code_scale
    psychoJS.experiment.addData("X Scale", x_scale);
    psychoJS.experiment.addData("Y Scale", y_scale);
    
    // the Routine "screen_scale" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    // Routines running outside a loop should always advance the datafile row
    if (currentLoop === psychoJS.experiment) {
      psychoJS.experiment.nextEntry(snapshot);
    }
    return Scheduler.Event.NEXT;
  }
}


var mouseDown;
var spotStart;
var spotSpeed;
var spotPos;
var spotSave;
var tStart;
var maxX;
var nPasses;
var blind_spot_RComponents;
function blind_spot_RRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //--- Prepare to start Routine 'blind_spot_R' ---
    t = 0;
    blind_spot_RClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // Run 'Begin Routine' code from code_spot
    mouseDown = false;
    spotStart = 0;
    spotSpeed = 3;
    spotPos = spotStart;
    spotSave = [];
    tStart = 3;
    maxX = (psychoJS.window.size[0] / 2);
    console.log("maxX", maxX);
    nPasses = 0;
    fixation_cross.setSize([x_scale, y_scale]);
    
    spot.setPos([((- x_scale) * 5), 0]);
    spot.setSize([(x_scale / 2), (y_scale / 2)]);
    head.setPos([((- 5) * x_scale), y_scale]);
    head.setSize([(2.5 * x_scale), (4 * y_scale)]);
    fixation_cross.setPos([((- 5) * x_scale), 0]);
    fixation_cross.setSize([x_scale, y_scale]);
    wink.setPos([((- 5) * x_scale), y_scale]);
    wink.setSize([(2.17 * x_scale), (1.58 * y_scale)]);
    // keep track of which components have finished
    blind_spot_RComponents = [];
    blind_spot_RComponents.push(text_top_R);
    blind_spot_RComponents.push(spot);
    blind_spot_RComponents.push(head);
    blind_spot_RComponents.push(fixation_cross);
    blind_spot_RComponents.push(wink);
    
    for (const thisComponent of blind_spot_RComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function blind_spot_RRoutineEachFrame() {
  return async function () {
    //--- Loop for each frame of Routine 'blind_spot_R' ---
    // get current time
    t = blind_spot_RClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // Run 'Each Frame' code from code_spot
    var _pj;
    function _pj_snippets(container) {
        function in_es6(left, right) {
            if (((right instanceof Array) || ((typeof right) === "string"))) {
                return (right.indexOf(left) > (- 1));
            } else {
                if (((right instanceof Map) || (right instanceof Set) || (right instanceof WeakMap) || (right instanceof WeakSet))) {
                    return right.has(left);
                } else {
                    return (left in right);
                }
            }
        }
        container["in_es6"] = in_es6;
        return container;
    }
    _pj = {};
    _pj_snippets(_pj);
    keys = psychoJS.eventManager.getKeys();
    if ((t > tStart)) {
        fixation_cross.setSize([(x_scale * (0.75 + (Math.sin((t * 4)) * 0.25))), (y_scale * (0.75 + (Math.sin((t * 6)) * 0.25)))]);
        if (keys.length) {
            if (_pj.in_es6("space", keys)) {
                spotSave.push(spotPos);
                fixation_cross.setFillColor(red);
                fixation_cross.setLineColor(red);
                mouseDown = true;
            }
        } else {
            if ((mouseDown === true)) {
                fixation_cross.setFillColor(white);
                fixation_cross.setLineColor(white);
                mouseDown = false;
            }
        }
        if ((spotPos > maxX)) {
            spotStart = maxX;
            spotSpeed = (- 3);
            tStart = (t - 0.1);
            nPasses += 1;
        } else {
            if ((spotPos < 0)) {
                spotStart = 0;
                spotSpeed = 3;
                tStart = (t - 0.1);
                nPasses += 1;
            }
        }
        spotPos = (spotStart + ((x_scale * spotSpeed) * (t - tStart)));
        spot.setPos([spotPos, 0]);
        if ((nPasses > 3)) {
            continueRoutine = false;
        }
    }
    
    
    // *text_top_R* updates
    if (t >= 0.0 && text_top_R.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text_top_R.tStart = t;  // (not accounting for frame time here)
      text_top_R.frameNStart = frameN;  // exact frame index
      
      text_top_R.setAutoDraw(true);
    }

    
    // *spot* updates
    if (t >= 0 && spot.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      spot.tStart = t;  // (not accounting for frame time here)
      spot.frameNStart = frameN;  // exact frame index
      
      spot.setAutoDraw(true);
    }

    
    // *head* updates
    if (t >= 0.0 && head.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      head.tStart = t;  // (not accounting for frame time here)
      head.frameNStart = frameN;  // exact frame index
      
      head.setAutoDraw(true);
    }

    
    // *fixation_cross* updates
    if (t >= 0.0 && fixation_cross.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      fixation_cross.tStart = t;  // (not accounting for frame time here)
      fixation_cross.frameNStart = frameN;  // exact frame index
      
      fixation_cross.setAutoDraw(true);
    }

    
    // *wink* updates
    if (t >= 0.0 && wink.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      wink.tStart = t;  // (not accounting for frame time here)
      wink.frameNStart = frameN;  // exact frame index
      
      wink.setAutoDraw(true);
    }

    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of blind_spot_RComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


var blind_R1;
var blind_R2;
function blind_spot_RRoutineEnd(snapshot) {
  return async function () {
    //--- Ending Routine 'blind_spot_R' ---
    for (const thisComponent of blind_spot_RComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // Run 'End Routine' code from code_spot
    blind_R1 = 0;
    blind_R2 = 0;
    if ((spotSave.length > 1)) {
        spotSave.sort();
        blind_R1 = (spotSave[Number.parseInt((spotSave.length * 0.1))] + (5 * x_scale));
        blind_R2 = (spotSave[Number.parseInt((spotSave.length * 0.9))] + (5 * x_scale));
        console.log("R1", blind_R1);
        console.log("R2", blind_R2);
    }
    
    // the Routine "blind_spot_R" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    // Routines running outside a loop should always advance the datafile row
    if (currentLoop === psychoJS.experiment) {
      psychoJS.experiment.nextEntry(snapshot);
    }
    return Scheduler.Event.NEXT;
  }
}


var blind_spot_LComponents;
function blind_spot_LRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //--- Prepare to start Routine 'blind_spot_L' ---
    t = 0;
    blind_spot_LClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // Run 'Begin Routine' code from code_spot_L
    mouseDown = false;
    spotStart = 0;
    spotSpeed = (- 3);
    spotPos = spotStart;
    spotSave = [];
    tStart = 3;
    maxX = (psychoJS.window.size[0] / 2);
    console.log("maxX", maxX);
    nPasses = 0;
    fixation_cross.setSize([x_scale, y_scale]);
    fixation_cross.setPos([(5 * x_scale), 0]);
    spot.setPos([(5 * x_scale), 0]);
    spot.setAutoDraw(true);
    fixation_cross.setAutoDraw(true);
    
    head_2.setPos([(5 * x_scale), (3 * y_scale)]);
    head_2.setSize([(2.5 * x_scale), (3.5 * y_scale)]);
    wink_2.setPos([(5 * x_scale), (3 * y_scale)]);
    wink_2.setSize([(2.17 * x_scale), (1.58 * y_scale)]);
    // keep track of which components have finished
    blind_spot_LComponents = [];
    blind_spot_LComponents.push(text_top_L);
    blind_spot_LComponents.push(head_2);
    blind_spot_LComponents.push(wink_2);
    
    for (const thisComponent of blind_spot_LComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function blind_spot_LRoutineEachFrame() {
  return async function () {
    //--- Loop for each frame of Routine 'blind_spot_L' ---
    // get current time
    t = blind_spot_LClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // Run 'Each Frame' code from code_spot_L
    var _pj;
    function _pj_snippets(container) {
        function in_es6(left, right) {
            if (((right instanceof Array) || ((typeof right) === "string"))) {
                return (right.indexOf(left) > (- 1));
            } else {
                if (((right instanceof Map) || (right instanceof Set) || (right instanceof WeakMap) || (right instanceof WeakSet))) {
                    return right.has(left);
                } else {
                    return (left in right);
                }
            }
        }
        container["in_es6"] = in_es6;
        return container;
    }
    _pj = {};
    _pj_snippets(_pj);
    keys = psychoJS.eventManager.getKeys();
    if ((t > tStart)) {
        fixation_cross.setSize([(x_scale * (0.75 + (Math.sin((t * 4)) * 0.25))), (y_scale * (0.75 + (Math.sin((t * 6)) * 0.25)))]);
        if (keys.length) {
            if (_pj.in_es6("space", keys)) {
                spotSave.push(spotPos);
                fixation_cross.setFillColor(red);
                fixation_cross.setLineColor(red);
                mouseDown = true;
            }
        } else {
            if ((mouseDown === true)) {
                fixation_cross.setFillColor(white);
                fixation_cross.setLineColor(white);
                mouseDown = false;
            }
        }
        if ((spotPos < (- maxX))) {
            spotStart = (- maxX);
            spotSpeed = 3;
            tStart = (t - 0.1);
            nPasses += 1;
        } else {
            if ((spotPos > 0)) {
                spotStart = 0;
                spotSpeed = (- 3);
                tStart = (t - 0.1);
                nPasses += 1;
            }
        }
        spotPos = (spotStart + ((x_scale * spotSpeed) * (t - tStart)));
        spot.setPos([spotPos, 0]);
        if ((nPasses > 3)) {
            continueRoutine = false;
        }
    }
    
    
    // *text_top_L* updates
    if (t >= 0.0 && text_top_L.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text_top_L.tStart = t;  // (not accounting for frame time here)
      text_top_L.frameNStart = frameN;  // exact frame index
      
      text_top_L.setAutoDraw(true);
    }

    
    // *head_2* updates
    if (t >= 0.0 && head_2.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      head_2.tStart = t;  // (not accounting for frame time here)
      head_2.frameNStart = frameN;  // exact frame index
      
      head_2.setAutoDraw(true);
    }

    
    // *wink_2* updates
    if (t >= 0.0 && wink_2.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      wink_2.tStart = t;  // (not accounting for frame time here)
      wink_2.frameNStart = frameN;  // exact frame index
      
      wink_2.setAutoDraw(true);
    }

    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of blind_spot_LComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


var blind_L1;
var blind_L2;
function blind_spot_LRoutineEnd(snapshot) {
  return async function () {
    //--- Ending Routine 'blind_spot_L' ---
    for (const thisComponent of blind_spot_LComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // Run 'End Routine' code from code_spot_L
    blind_L1 = 0;
    blind_L2 = 0;
    if ((spotSave.length > 1)) {
        spotSave.sort();
        blind_L1 = (spotSave[Number.parseInt((spotSave.length * 0.1))] - (5 * x_scale));
        blind_L2 = (spotSave[Number.parseInt((spotSave.length * 0.9))] - (5 * x_scale));
        console.log("L1", blind_L1);
        console.log("L2", blind_L2);
    }
    spot.setAutoDraw(false);
    fixation_cross.setAutoDraw(false);
    
    // the Routine "blind_spot_L" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    // Routines running outside a loop should always advance the datafile row
    if (currentLoop === psychoJS.experiment) {
      psychoJS.experiment.nextEntry(snapshot);
    }
    return Scheduler.Event.NEXT;
  }
}


var visual_angle;
var _key_resp_allKeys;
var rectangleComponents;
function rectangleRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //--- Prepare to start Routine 'rectangle' ---
    t = 0;
    rectangleClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // Run 'Begin Routine' code from code
    fixation_cross.setPos([0, 0]);
    fixation_cross.setColor(blue);
    fixation_cross.setAutoDraw(true);
    circle_10.setFillColor(transparent);
    visual_angle = ((((blind_R1 + blind_R2) - blind_L1) - blind_L2) / 4);
    console.log("visual_angle 15 degrees", visual_angle);
    visual_angle = (visual_angle / 15);
    console.log("visual_angle pixels", visual_angle);
    visual_angle = (visual_angle / x_scale);
    console.log("visual_angle cm", visual_angle);
    
    polygon.setSize([(5 * x_scale), (5 * y_scale)]);
    key_resp.keys = undefined;
    key_resp.rt = undefined;
    _key_resp_allKeys = [];
    spot_L.setPos([((blind_L1 + blind_L2) / 2), ((blind_L1 + blind_L2) / 20)]);
    spot_L.setSize([(blind_L2 - blind_L1), (1.3 * (blind_L2 - blind_L1))]);
    spot_R.setPos([((blind_R1 + blind_R2) / 2), ((- (blind_R1 + blind_R2)) / 20)]);
    spot_R.setSize([(blind_R2 - blind_R1), (1.3 * (blind_R2 - blind_R1))]);
    circle_10.setSize([((x_scale * visual_angle) * 10), ((y_scale * visual_angle) * 10)]);
    // keep track of which components have finished
    rectangleComponents = [];
    rectangleComponents.push(text);
    rectangleComponents.push(polygon);
    rectangleComponents.push(key_resp);
    rectangleComponents.push(spot_L);
    rectangleComponents.push(spot_R);
    rectangleComponents.push(circle_10);
    
    for (const thisComponent of rectangleComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function rectangleRoutineEachFrame() {
  return async function () {
    //--- Loop for each frame of Routine 'rectangle' ---
    // get current time
    t = rectangleClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // Run 'Each Frame' code from code
    fixation_cross.setSize([((5 * x_scale) * (0.75 + (Math.sin((t * 4)) * 0.25))), ((5 * y_scale) * (0.75 + (Math.sin((t * 6)) * 0.25)))]);
    
    
    // *text* updates
    if (t >= 0.0 && text.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text.tStart = t;  // (not accounting for frame time here)
      text.frameNStart = frameN;  // exact frame index
      
      text.setAutoDraw(true);
    }

    
    // *polygon* updates
    if (t >= 0.0 && polygon.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      polygon.tStart = t;  // (not accounting for frame time here)
      polygon.frameNStart = frameN;  // exact frame index
      
      polygon.setAutoDraw(true);
    }

    
    // *key_resp* updates
    if (t >= 0.0 && key_resp.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      key_resp.tStart = t;  // (not accounting for frame time here)
      key_resp.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { key_resp.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { key_resp.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { key_resp.clearEvents(); });
    }

    if (key_resp.status === PsychoJS.Status.STARTED) {
      let theseKeys = key_resp.getKeys({keyList: ['space'], waitRelease: false});
      _key_resp_allKeys = _key_resp_allKeys.concat(theseKeys);
      if (_key_resp_allKeys.length > 0) {
        key_resp.keys = _key_resp_allKeys[_key_resp_allKeys.length - 1].name;  // just the last key pressed
        key_resp.rt = _key_resp_allKeys[_key_resp_allKeys.length - 1].rt;
        key_resp.duration = _key_resp_allKeys[_key_resp_allKeys.length - 1].duration;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    
    // *spot_L* updates
    if (t >= 0.0 && spot_L.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      spot_L.tStart = t;  // (not accounting for frame time here)
      spot_L.frameNStart = frameN;  // exact frame index
      
      spot_L.setAutoDraw(true);
    }

    
    // *spot_R* updates
    if (t >= 0.0 && spot_R.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      spot_R.tStart = t;  // (not accounting for frame time here)
      spot_R.frameNStart = frameN;  // exact frame index
      
      spot_R.setAutoDraw(true);
    }

    
    // *circle_10* updates
    if (t >= 0.0 && circle_10.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      circle_10.tStart = t;  // (not accounting for frame time here)
      circle_10.frameNStart = frameN;  // exact frame index
      
      circle_10.setAutoDraw(true);
    }

    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of rectangleComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function rectangleRoutineEnd(snapshot) {
  return async function () {
    //--- Ending Routine 'rectangle' ---
    for (const thisComponent of rectangleComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // Run 'End Routine' code from code
    fixation_cross.setAutoDraw(false);
    
    key_resp.stop();
    // the Routine "rectangle" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    // Routines running outside a loop should always advance the datafile row
    if (currentLoop === psychoJS.experiment) {
      psychoJS.experiment.nextEntry(snapshot);
    }
    return Scheduler.Event.NEXT;
  }
}


var endComponents;
function endRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //--- Prepare to start Routine 'end' ---
    t = 0;
    endClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // keep track of which components have finished
    endComponents = [];
    endComponents.push(text_end);
    
    for (const thisComponent of endComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function endRoutineEachFrame() {
  return async function () {
    //--- Loop for each frame of Routine 'end' ---
    // get current time
    t = endClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *text_end* updates
    if (t >= 0.0 && text_end.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text_end.tStart = t;  // (not accounting for frame time here)
      text_end.frameNStart = frameN;  // exact frame index
      
      text_end.setAutoDraw(true);
    }

    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of endComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function endRoutineEnd(snapshot) {
  return async function () {
    //--- Ending Routine 'end' ---
    for (const thisComponent of endComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // the Routine "end" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    // Routines running outside a loop should always advance the datafile row
    if (currentLoop === psychoJS.experiment) {
      psychoJS.experiment.nextEntry(snapshot);
    }
    return Scheduler.Event.NEXT;
  }
}


function importConditions(currentLoop) {
  return async function () {
    psychoJS.importAttributes(currentLoop.getCurrentTrial());
    return Scheduler.Event.NEXT;
    };
}


async function quitPsychoJS(message, isCompleted) {
  // Check for and save orphaned data
  if (psychoJS.experiment.isEntryEmpty()) {
    psychoJS.experiment.nextEntry();
  }
  
  
  
  
  
  
  
  
  
  
  psychoJS.window.close();
  psychoJS.quit({message: message, isCompleted: isCompleted});
  
  return Scheduler.Event.QUIT;
}
